/*****************************************************
**********                                  **********
**********  PROGRAM: Century        		**********
**********			 Line Guided Robot		**********
**********  Version: 1.6                    **********
**********  BC.SOLARSKI (C) 2008/04         **********
**********  solarskit.wz.cz                 **********
**********  solarskit@gmail.com             **********
**********  ATmega8-16PU                    **********
**********  XTAL = 16,0000MHz               **********
**********  t = 62,5ns                      **********
**********					    	        **********
**********	last revision:                  **********
**********	18.04.2008 2205                 **********
**********                                  **********
*****************************************************/

/******************************************************************************

	BAUD:
	---------------------------------
	56700 1sp no parity
	---
	AT\r - get the OK response
	ID\r - get the identifikation
	VER\r - get current version of firmware
	AUTOR\r - get creator's name
	---------------------------------
	
******************************************************************************/

/******************************************************************************
	Addition Required Headers/Libraries
******************************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <string.h>
#include <stdlib.h>
#include <avr/eeprom.h>
#include <avr/wdt.h>
#include <avr/pgmspace.h>

/******************************************************************************
	Compiler Constants
******************************************************************************/

// 56700 baudrate
#define MYUBRR 34


// reference for ADC in mV
//Vref is 2.456V and ratio 10k/(10k+15k) is 0.4
#define VREF 6140

// number of samples gained for measuring voltage
#define SAMPLES_COUNT 100

// period of RUN LED signalization blinking in ms
#define TIMER_RUN_MAX 150

// regulator sample time in ms
#define TIME_SAMPLE 5

// key bouncing terminate
#define TIME_CHIRP 250

// period och sonar distance measure
#define TIME_DISTANCE 50

// period of line following
#define TIME_LINE_FOLLOW 2

#define TIME_VOLT 1000

#define TIME_START_BEGIN 1000

// EEPROM adress for PID constant storage
#define ADRESS_P 0
#define ADRESS_I 1
#define ADRESS_D 2

// --- MCU's PINS ---
// multipourpose signalization LEDs - outs
#define LED0 PB0
#define LED1 PB1
#define LED2 PB2
#define LED3 PB3
// program run signalization - out
#define LED4 PB4
// battery pack voltage measure input - input
#define VBAT PA7

// start button - input
#define START PA6

// direction signals from quadrature sensors in Faulhaber motor - inputs
#define QDIRL PA2
#define QDIRR PA3

// set the motor direction - outs
#define DIRL PD6
#define DIRR PD7

// sonar distance measure
#define ECHO PA4
#define TRIG PA5

/******************************************************************************
	Variables Definition And Declaration
******************************************************************************/

// --- variables to handle an USART  events ---
// field, called buffer, which contains received characters
volatile	uint8_t		buffer[] = "................";
// pointer in a buffer
volatile	uint8_t		buffCt = 0;
// parameter
volatile 	uint8_t 	* parameter;
// separated parameter
volatile	uint32_t		parSep1, parSep2, parSep3;

// timer to run LED blinking signalization
volatile	uint16_t	TimerRun;

// --- PSD REGULATOR ---
// tics counters, current and last values
volatile	int32_t		countL = 0;
volatile	int32_t		count0L = 0;
volatile	int32_t		countR = 0;
volatile	int32_t		count0R = 0;
// speeds ( count/time ), current and last values
volatile	int32_t		speedL = 0;
volatile	int32_t		speedR = 0;
volatile	int32_t		speed0L = 0;
volatile	int32_t		speed0R = 0;
// demanded speed
volatile	int32_t		speedDemandL = 0;
volatile	int32_t		speedDemandR = 0;
// regulation errors ( deviation )
volatile	int32_t		eL =0;
volatile	int32_t		eR =0;
// action value ( PWM )
volatile	int32_t		AVLeft =0;
volatile	int32_t		AVRight =0;
// storage of action value from I part from regulator
volatile	int32_t		storeIL =0;
volatile	int32_t		storeIR =0;

// --- regulation constants ---
volatile 	int32_t		P = 10;
volatile 	int32_t		I = 25;
volatile 	int32_t		D = 5;

// sample timer
volatile	uint8_t		TimerSample;
// run regulation alghoritm flag
volatile	uint8_t		regulFlag;

// timer to eliminate bouncing of key
volatile	uint32_t	TimerChirp;

// measured voltage
volatile	uint16_t	volt;
// register used to calculate average - sum
volatile	uint32_t	voltSum;
// ADC samples counter
volatile	uint8_t		samplesCt;

// line following alghoritmus running
volatile	uint8_t		LineRunFlag;

// timer to sonar distance measure
volatile	uint16_t	TimerSonar;
// flag to measure impulse lenght from sonar, trigger was sent
volatile	uint8_t		TriggerFlag;
// store registerd of echo input, now and past
volatile	uint8_t		Echo, Echo0;
// measured distance in mm
volatile	uint16_t	Distance;
// sonar distance measure period timer
volatile	uint16_t	TimerDistance;

//
volatile	int32_t countStep0L;
volatile	int32_t countStep0R;

volatile	uint32_t Count;

volatile	uint8_t MoveFlag;

volatile	uint8_t MoveDoneFlag = 1;

volatile	uint16_t LineFollowTimeFlag;
volatile	uint16_t TimerLineFollow;

volatile	uint16_t TimerVolt;

//
volatile	uint8_t ObstaclePhase;

//
volatile	uint8_t FindLineFlag;

//
volatile	uint8_t TurnFlag;

//
volatile	uint8_t  StartBeginFlag;
volatile	uint16_t StartBeginTimer;


/******************************************************************************
	Functions
******************************************************************************/

//---------------------------------------------------------------------- >> Sonar sensor <<
// trigger sonar
void Trigger( void )
{
	// clear timer 0
	TCNT0 = 0;
	// trigger impulse - 10 us
	PORTA |= ( 1 << TRIG );
	// CLKio / 8
	TCCR0 |= ( 1 << CS01 );
	// wait 10us
	while( TCNT0 < 20 )
		;
	// stop timer
	TCCR0 &= ~( 1 << CS01 );
	// low the trigger impulse
	PORTA &= ~( 1 << TRIG );
	// set flag
	TriggerFlag = 1;

}

//---------------------------------------------------------------------- >> Sonar sensor <<
// measure echo impulse
void CatchEcho( void )
{
	// read Echo pin
	Echo = PINA & ( 1 << ECHO );
	// rise edge
	if ( Echo && !( Echo0 ) )
	{
		// erase overflow counter
		TimerSonar = 0;
		// erase counter
		TCNT0 = 0;
		// run timer 0 - CLKio / 8
		TCCR0 |= ( 1 << CS01 );
	}
	// falling edge
	if ( !( Echo ) && Echo0 )
	{
		// stop timer
		TCCR0 &= ~( 1 << CS01 );
		// calculate distance
		Distance = TimerSonar * 8 ;
		// deactivate flag
		TriggerFlag = 0;
	}
	// value in step - 1
	Echo0 = Echo;
}

//---------------------------------------------------------------------- >> Move Distance <<
//
void Move( int32_t Count1 , int8_t Left , int8_t Right )
{

	MoveDoneFlag = 0;

	countStep0L = countL;
	countStep0R = countR;

	Count = Count1;

	speedDemandL = Left;
	speedDemandR = Right;

	MoveFlag = 1;
}

//---------------------------------------------------------------------- >> Move Distance <<
//
void MoveSteps( void )
{
	if ( ( ( countL - countStep0L + countR - countStep0R ) / 2 ) >= Count )
	{

		MoveDoneFlag = 1;

		if ( ObstaclePhase != 0 )
		{

			FindLineFlag = 1;

			if ( ++ObstaclePhase > 3 )
			{	
				ObstaclePhase = 0;
				
				speedDemandL = 40;
				speedDemandR = 40;

				MoveFlag = 0;

				//FindLineFlag = 1;
			}			
		}
		else
		{
			speedDemandL = 0;
			speedDemandR = 0;

			MoveFlag = 0;

			if ( TurnFlag )
			{
				TurnFlag = 0;
				FindLineFlag = 0;
			}
		}
	}
}

//---------------------------------------------------------------------- >> Run Obstacle <<
//
void RunObstacle( void )
{
	
	switch ( ObstaclePhase )
	{
		// phase 1 
	case 1:
		//Move( 1800 , 65 , 10 );
		Move( 1800 , 10 , 65 );
		break;
		// phase 2 
	case 2:
		Move( 500 , 65 , 65 );
		break;
		// phase 3 
	case 3:
		//Move( 11000 , 42 , 65 );
		Move( 11000 , 65 , 42 );
		break;
	}
}

//---------------------------------------------------------------------- >> Line sensor <<
// read data from line sensor
uint8_t Line( void )
{
	return PINC;
}

//---------------------------------------------------------------------- >> Find Line <<
//
void FindLine( void )
{
	
	if ( Line() & 0b00000010 )
	{
		FindLineFlag = 0;
		ObstaclePhase = 0;
		//TurnFlag = 1;
		//Move( 1200 , 65 , 10 );
		Move( 1200 , 10 , 65 );
	}
}

//---------------------------------------------------------------------- >> Comunication by USART <<
// transmition a byte ( 8 bits )
void sendByte( uint8_t data )
{

	// wait until transmit register is empty
	while ( !( UCSRA & ( 1 << UDRE ) ) );
	// send data
	UDR = data;

}

//---------------------------------------------------------------------- >> Comunication by USART <<
// transmition a number in ASCII
void sendNum( int16_t num )
// count je pocet zabranzch pozic
{
	uint8_t s_num[ 10 ];
	uint8_t j;
	itoa( num , s_num , 10 );
	j = 0;
	while ( s_num[ j ] != '\0' )
	{
		sendByte( s_num[ j++ ] );
	}
}

//---------------------------------------------------------------------- >> Comunication by USART <<
// 
void SendText( const char *addr )
{
  char c;

  while ( (c = pgm_read_byte( addr++ ) ) )
    sendByte( c );
}

//---------------------------------------------------------------------- >> Handle of USART <<
// USART Text handler
void USARThandle( void )
{

	// AT
	if ( strcasestr( buffer , "at" ) != NULL )
		SendText( PSTR( ">OK\r" ) );

	// ID
	if ( strcasestr( buffer , "id" ) != NULL )
		SendText( PSTR( ">CENTURY, Line Guided Robot with ATmega16\r" ) );

	// VER
	if ( strcasestr( buffer , "ver" ) != NULL )
		SendText( PSTR( ">CENTURY V1.5 2008\r" ) );

	// AUTOR
	if ( strcasestr( buffer , "autor" ) != NULL )
		SendText( PSTR(">Bc. Tomas Solarski, www.solarskit.wz.cz\r") );
	
	// VOLT
	if ( strcasestr( buffer , "volt" ) != NULL )
	{
		SendText( PSTR( ">Measured voltage: " ) );
		sendNum( volt );
		SendText( PSTR( "\r" ) );
	}
	// COUNT
	if ( strcasestr( buffer , "count" ) != NULL )
	{
		SendText( PSTR( ">Increment Counters:\r L: " ) );
		sendNum( countL );
		SendText( PSTR( "\tR: " ) );
		sendNum( countR );
		SendText( PSTR( "\r" ) );
	}
	// LINE
	if ( strcasestr( buffer , "line" ) != NULL )
	{
		SendText( PSTR( ">Line Sensor: " ) );
		sendNum( Line() );
		SendText( PSTR( "\r" ) );
	}
	// DIAG
	if ( strcasestr( buffer , "diag" ) != NULL )
	{
		SendText( PSTR(  ">Diag:\r" ) );
		sendNum( speedL );
		SendText( PSTR(  "\t" ) );
		sendNum( speedDemandL );
		SendText( PSTR(  "\t" ) );
		sendNum( eL );
		SendText( PSTR(  "\t" ) );
		sendNum( AVLeft );
		SendText( PSTR(  "\r" ) );
	}

	// DIST
	if ( strcasestr( buffer , "dist" ) != NULL )
	{
		SendText( PSTR( ">Distance:\r" ) );
		sendNum( Distance );
		SendText( PSTR( "\r" ) );
	}

	// LEFT
	if ( strcasestr( buffer , "left" ) != NULL )
	{
		// Left Wheel Speed command ack
		SendText( PSTR( ">Left Speed\r" ) );
		// behind the space character there is a parameter
		parameter = strcasestr( buffer , " " );
		// convert ASCII number into integer
		speedDemandL = atoi( parameter + 1 );
	}

	// RIGHT
	if ( strcasestr( buffer , "right" ) != NULL )
	{
		// Left Wheel Speed command ack
		SendText( PSTR( ">Right Speed\r" ) );
		// behind the space character there is a parameter
		parameter = strcasestr( buffer , " " );
		// convert ASCII number into integer
		speedDemandR = atoi( parameter + 1 );
	}

	// PGAIN
	if ( strcasestr( buffer , "pgain" ) != NULL )
	{
		// P constant ack
		SendText( PSTR( ">P ACK\r" ) );
		// behind the space character there is a parameter
		parameter = strcasestr( buffer , " " );
		// convert ASCII number into integer
		P = atoi( parameter );
		// save to eeprom
		eeprom_write_byte( ADRESS_P , P );
	}

	// IGAIN
	if ( strcasestr( buffer , "igain" ) != NULL )
	{
		// I constant ack
		SendText( PSTR( ">I ACK\r" ) );
		// behind the space character there is a parameter
		parameter = strcasestr( buffer , " " );
		// convert ASCII number into integer
		I = atoi( parameter );
		// save to eeprom
		eeprom_write_byte( ADRESS_I , I );
	}

	// DGAIN
	if ( strcasestr( buffer , "dgain" ) != NULL )
	{
		// D constant ack
		SendText( PSTR( ">D ACK\r" ) );
		// behind the space character there is a parameter
		parameter = strcasestr( buffer , " " );
		// convert ASCII number into integer
		D = atoi( parameter );
		// save to eeprom
		eeprom_write_byte( ADRESS_D , D );
	}

	// PID
	if ( strcasestr( buffer , "pid" ) != NULL )
	{
		SendText( PSTR( ">PID constants:\r" ) );
		sendNum( P );
		SendText( PSTR( "\t" ) );
		sendNum( I );
		SendText( PSTR( "\t" ) );
		sendNum( D );
		SendText( PSTR( "\r" ) );
	
	}

	// MOVE
	if ( strcasestr( buffer , "move" ) != NULL )
	{
		SendText( PSTR( ">Move Distance\r" ) );
		// behind the space character there is a parameter
		parameter = strcasestr( buffer , "!" );
		parSep1 = atol( parameter + 1 );
		parameter = strcasestr( buffer , "@" );
		parSep2 = atoi( parameter + 1 );
		parameter = strcasestr( buffer , "#" );
		parSep3 = atoi( parameter + 1 );

		Move( parSep1 , parSep2 , parSep3 );
	}

	// POM
	if ( strcasestr( buffer , "pom" ) != NULL )
	{
		SendText( PSTR( ">Move Distance 2\r" ) );
		Move( 1000 , 10 , 10 );
	}

	// 
	if ( strcasestr( buffer , "obstacle" ) != NULL )
	{
		SendText( PSTR( ">Obstacle\r" ) );

		ObstaclePhase = 1;

		RunObstacle();
	}


	// set pointer to the first char
	buffCt = 0;
	
	// erase buffer
	uint8_t i = 0;

	while ( buffer[ i ] != '\0' )
		buffer[ i++ ] = '.';

}

//---------------------------------------------------------------------- >> START Button <<
// read value from start button pin
uint8_t Start( void )
{

	if ( PINA & ( 1 << START ) )
		return 0;
	else
		return 1;

}

//---------------------------------------------------------------------- >> Regulation <<
void Regul ( void )
{

	// regulation errors
	eL = speedDemandL - speedL;
	eR = speedDemandR - speedR;

	// ----- ===== ::::: RIGHT WHEEL ::::: ===== -----

	// P
	AVRight = P * eR;
	// I
	storeIR = storeIR + ( I * eR);
	AVRight = AVRight + storeIR;
	// D
	AVRight = AVRight + D * ( speed0R - speedR);

	AVRight /= 10;

	// action value limits
	if ( AVRight >= 399 )
		AVRight=ICR1;
	if ( AVRight <= -399 )
		AVRight = -399;

	// action value apply to motor
	if ( AVRight >=0 )
	{
		// run forward
	 	PORTD &= ~( 1 << DIRR );
		// PWM
	 	OCR1A = AVRight;
	}
	else
	{
		// run reverse
	 	PORTD |= ( 1 << DIRR );
		// PWM
	 	OCR1A = ICR1 + AVRight;
	}

	// store count in this step
	count0R = countR;

	// I limitation - wind up
	if ( storeIR >= 50000 )
		storeIR = 50000;
	if ( storeIR <= -50000 )
		storeIR = -50000;

	// speed in this step
	speed0R = speedR;


	// ----- ===== ::::: LEFT WHEEL ::::: ===== -----

	// P
	AVLeft = P * eL;
	// I
	storeIL = storeIL + ( I * eL);
	AVLeft = AVLeft + storeIL;
	// D
	AVLeft = AVLeft + D * ( speed0L - speedL);

	AVLeft /= 10;

	// action value limits
	if ( AVLeft >= 399 )
		AVLeft=ICR1;
	if ( AVLeft <= -399 )
		AVLeft = -399;

	// action value apply to motor
	if ( AVLeft >=0 )
	{
		// run forward
	 	PORTD |= ( 1 << DIRL );
		// PWM
	 	OCR1B = ICR1 - AVLeft;
	}
	else
	{
		// run reverse
	 	PORTD &= ~( 1 << DIRL );
		// PWM
	 	OCR1B = -AVLeft;
	}

	// store count in this step
	count0L = countL;

	//I limitation - wind up
	if ( storeIL >= 50000 )
		storeIL = 50000;
	if ( storeIL <= -50000 )
		storeIL = -50000;

	// speed in this step
	speed0L = speedL;

	// deactivate flag
	regulFlag=0;
}

//---------------------------------------------------------------------- >> Line Following <<
void LineRun ( void )
{

	switch ( Line() )
	{
		LineFollowTimeFlag = 0;

		case 0b10000000:
		case 0b11000000:
		case 0b11100000:
		case 0b11110000:
			speedDemandL = 5;
			speedDemandR = 90;
			break;

		case 0b00000001:
		case 0b00000011:
		case 0b00000111:
		case 0b00001111:
			speedDemandL = 90;
			speedDemandR = 5;
			break;

		case 0b01000000:
		case 0b01100000:
		case 0b01110000:
			speedDemandL = 40;
			speedDemandR = 80;
			break;

		case 0b00000010:
		case 0b00000110:
		case 0b00001110:
			speedDemandL = 80;
			speedDemandR = 40;
			break;
	
		case 0b00100000:
		case 0b00110000:
		case 0b00111000:
			speedDemandL = 62;
			speedDemandR = 80;
			break;
	
		case 0b00000100:
		case 0b00001100:
		case 0b00011100:
			speedDemandL = 80;
			speedDemandR = 62;
			break;

		case 0b00010000:
			speedDemandL = 62;
			speedDemandR = 75;
			break;

		case 0b00001000:
			speedDemandL = 75;
			speedDemandR = 62;
			break;

		case 0b00011000:
		case 0b00111100:		
			speedDemandL = 62;
			speedDemandR = 62;
			break;

		case 0b00000000:
			speedDemandL = 45;
			speedDemandR = 45;
			break;
		case 0b11111111:
			speedDemandL = 0;
			speedDemandR = 0;
			break;
		default:
			speedDemandL = 35;
			speedDemandR = 35;
	}

		

}

/******************************************************************************
	MAIN Function
******************************************************************************/

int main( void )
{

//---------------------------------------------------------------------- >> I/O <<
//0 - INPUT, 1 - OUTPUT

							//	.0		.1		.2		.3		.4		.5		.6		.7
	DDRA	= 0b00100000;	//	-		-		QDIRL	QDIRR	ECHO	TRIG	START	VBAT 
	PORTA	= 0b01011100;
	DDRB	= 0b00011111;	//	LED0	LED1	LED2	LED3	LED4	MOSI	MISO	SCK 
	PORTB	= 0b00000000;
	DDRC	= 0b00000000;	//	LN0		LN1		LN2		LN3		LN4		LN5		LN6		LN7
	PORTC	= 0b00000000;
	DDRD	= 0b11110000;	//	RXD		TXD		IRQR	IRQL	PWML	PWMR	DIRL	DIRR
	PORTD	= 0b00000000;

//---------------------------------------------------------------------- >> T/C0 <<
// Sonar
	// mode 2 - CTC
	TCCR0	|= ( 1 << WGM01 ) | ( 0 << WGM00 );
	// top for 50us
	OCR0	= 99;
	// CLKio / 8
	//TCCR0 |= ( 1 << CS01 );

	// interrupt enable from OC
	TIMSK	|= ( 1 << OCIE0 );

//---------------------------------------------------------------------- >> T/C1 <<
// PWM
	// mode 8 - PWM, Phase and Frequency Correct - top IR1
	TCCR1B	|= ( 1 << WGM13 ) | ( 0 << WGM12 );
	TCCR1A	|= ( 0 << WGM11 ) | ( 0 << WGM10 );
	// Clear OC1A/OC1B on Compare Match when upcounting. Set OC1A/OC1B on Compare Match when downcounting.
	TCCR1A	|= ( 1 << COM1A1 ) | ( 0 << COM1A0 );
	TCCR1A	|= ( 1 << COM1B1 ) | ( 0 << COM1B0 );
	// PWM Top - 20kHz
	ICR1	= 399;
	// PWM
	OCR1A	= 0;
	OCR1B	= 0;
	// CLKi/o / 1
	TCCR1B	|=  ( 0 << CS12 ) | ( 0 << CS11 ) | ( 1 << CS10 );


//---------------------------------------------------------------------- >> T/C2 <<
// 1ms
	// CTC - mode 2
	TCCR2	|= ( 1 << WGM21 ) | ( 0 << WGM20 );
	// Top
	OCR2	= 124;
	// clk / 128
	TCCR2	|= ( 1 << CS22 ) | ( 0 << CS21 ) | ( 1 << CS20 );
	// interrupt enable from OC
	TIMSK	|= ( 1 << OCIE2 );

//---------------------------------------------------------------------- >> ADC <<
// used to measure main power supply
	// AD reference is internal 2.54V
	ADMUX	|= ( 1 << REFS1 ) | ( 1 << REFS0 );
	// ADC7
	ADMUX	|= ( 0 << MUX3 ) | ( 1 << MUX2) | ( 1 << MUX1) | ( 1 << MUX0);
	// Enable ADC
	ADCSRA	|= ( 1 << ADEN );
	// Enable Interrupt
	ADCSRA	|= ( 1 << ADIE );
	// CLKi/o / 128 = 156.25 kHz
	ADCSRA	|= ( 1 << ADPS2 ) | ( 1 << ADPS1 ) | ( 1 << ADPS0 );
	// Start conversion
	ADCSRA	|= ( 1 << ADSC );

//---------------------------------------------------------------------- >> USART <<
//
	// the speed of data interchage will be doubled
	UCSRA |= ( 1 << U2X );
	// enable the receiver and transmitter unit
	UCSRB |= ( 1 << RXEN ) | ( 1 << TXEN);
	// enable an interruption from receiver
	UCSRB |= ( 1 << RXCIE );
	// set up mode of operation - 8-bit, no parity, 1 stop bit
	UCSRC |= ( 1 << URSEL ) | ( 3 << UCSZ0 );
	// baud rate set up, it is depend on XTAL, U2X bit and UBRR register
	UBRRH |= MYUBRR >> 8;
	UBRRL |= MYUBRR & 0x00FF;

//---------------------------------------------------------------------- >> IRQ <<
// speed measurement
	// rising edge for INT0 and INT1
	MCUCR |= ( 1 << ISC11 ) | ( 1 << ISC10 );
	MCUCR |= ( 1 << ISC01 ) | ( 1 << ISC00 );
	// enable interrupt from INT0 a INT1
	GICR |= ( 1 << INT1 ) | ( 1 << INT0 );		

	// global interrupt enable
	sei();
	
	// load PID constants
	P = eeprom_read_byte( ADRESS_P );
	I = eeprom_read_byte( ADRESS_I );
	D = eeprom_read_byte( ADRESS_D );

	// program runnig indicator
	PORTB |= ( 1 << LED4 );
	
	// send text across UART
	SendText( PSTR( ">Tomas SOLARSKI, Bc. - Development & Control\n>HARD ADVANCED ROBOTICS\n> (c)2008\n" ) );

/******************************************************************************
	INFINITE LOOP
******************************************************************************/
	while( 1 )
	{

		// ***** REGULATION *****		
		if ( regulFlag )
			Regul();
		
		// ***** LINE RUNNING *****
		if ( LineRunFlag && LineFollowTimeFlag && ( ObstaclePhase == 0 ) && ( FindLineFlag == 0 ) && MoveDoneFlag )
			LineRun();

		
		// ***** SONAR *****
		if ( TriggerFlag )
			CatchEcho();

		if ( ( Distance < 150 ) && LineRunFlag && ( ObstaclePhase == 0 ) )
		{
			ObstaclePhase = 1;			
			RunObstacle();
		}

		// ***** MOVE *****
		if ( MoveFlag )
			MoveSteps();

		// ***** OBSTACLE *****
		if ( ( ObstaclePhase != 0 ) && ( MoveDoneFlag ) )
			RunObstacle();

		// ***** FIND LINE *****
		if ( FindLineFlag )
			FindLine();

		// ***** TURN *****
		if ( TurnFlag );
		{
			//Move( 512 , 40 , 0 );
			//MoveDoneFlag = 0;
			TurnFlag = 0;
		}

		// ***** USART *****
		if ( buffer[ buffCt - 1 ] == '\r' )
			USARThandle();
	
		// ***** START KEY *****
		if ( Start() & ( TimerChirp > TIME_CHIRP ) )
		{
						
			TimerChirp = 0;
			StartBeginFlag = 1;
			//LineRunFlag = 1;
			PORTB |= ( 1 << LED3 );
		}

		// ***** START KEY PREASED AGAIN *****
		if ( Start() && LineRunFlag && ( TimerChirp > TIME_CHIRP ) )
		{

			TimerChirp = 0;

			speedDemandL = 0;
			speedDemandR = 0;

			LineRunFlag = 0;

			PORTB &= ~( 1 << LED3 );

		}

		// 
		if ( ( volt < 4500 ) && ( TimerVolt > TIME_VOLT ) )
		{
			PORTB |= ( 1 << LED1 );

			speedDemandL = 0;
			speedDemandR = 0;

			LineRunFlag = 0;
		}
	}
}

/******************************************************************************
	INTERRUPTION HANDLER ROUTINES
******************************************************************************/

//---------------------------------------------------------------------- >> INT0 <<
// right wheel
SIGNAL ( SIG_INTERRUPT0 )
{
	// increment or decrement in depend of direction
	if ( PINA & ( 1 << QDIRR ) )
		++countR;
	else
		--countR;
}

//---------------------------------------------------------------------- >> INT1 <<
// left wheel
SIGNAL ( SIG_INTERRUPT1 )
{
	// increment or decrement in depend of direction
	if ( PINA & ( 1 << QDIRL ) )
		--countL;
	else
		++countL;
}

//---------------------------------------------------------------------- >> ADC Conversion Complete <<
// voltage measure
SIGNAL ( SIG_ADC )
{

	voltSum += ADC;

	samplesCt++;

}

//---------------------------------------------------------------------- >> Interrupt from T/C2 <<
// period 10us

SIGNAL ( SIG_OUTPUT_COMPARE0 )
{

	PORTB = PINB ^ ( 1 << LED0 );

	TimerSonar++;

}

//---------------------------------------------------------------------- >> Interrupt from T/C2 <<
// period 1ms

SIGNAL ( SIG_OUTPUT_COMPARE2 )
{

	if ( StartBeginFlag )	
	{
		if ( ++StartBeginTimer > TIME_START_BEGIN )
		{
			StartBeginTimer = 0;
			StartBeginFlag = 0;
			LineRunFlag = 1;
		}
	}

	if ( ++TimerLineFollow >= TIME_LINE_FOLLOW )
	{
		TimerLineFollow = 0;
		
		LineFollowTimeFlag = 1;

	}

	if ( ++TimerSample >= TIME_SAMPLE )
	{
		TimerSample = 0;

		PORTB = PINB ^ ( 1 << LED2 );

		// speed calculations
		speedL = countL - count0L;
		speedR = countR - count0R;

		regulFlag = 1;

	}

	if ( ++TimerDistance >= TIME_DISTANCE )
	{
		TimerDistance = 0;
		Trigger();
	}

	TimerChirp++;


	// run led signalization blinking
	if ( ++TimerRun >= TIMER_RUN_MAX / 2)
	{
		TimerRun = 0;
		PORTB = PINB ^ ( 1 << LED4 );
	}

	// voltage measurement
	if ( samplesCt >= SAMPLES_COUNT )
	{
		// reset counter
		samplesCt = 0;
		// voltage calculation equation - sum divided by count of samples give average value
		volt = ( uint32_t )VREF * voltSum /  SAMPLES_COUNT / 1024;
		// sum register erase
		voltSum = 0;

	}

	

	// start next conversion	
	ADCSRA |= ( 1 << ADSC );

}

//---------------------------------------------------------------------- >> interrupt from USART receiver <<
// an interruption is generated each time when the character is received
SIGNAL ( SIG_UART_RECV )
{

	// save received char into a buffer, and increment the pointer, so next received char will be saved to the next cell
	buffer[ buffCt ] = UDR;

	if ( ++buffCt > 16 )
		buffCt = 0;

}
